# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: gateway/ratelimit.proto
# Protobuf Python Version: 5.29.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    0,
    '',
    'gateway/ratelimit.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ..buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from . import options_pb2 as gateway_dot_options__pb2
from ..google.api import field_behavior_pb2 as google_dot_api_dot_field__behavior__pb2
from ..google.api import resource_pb2 as google_dot_api_dot_resource__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x17gateway/ratelimit.proto\x12\x07gateway\x1a\x1b\x62uf/validate/validate.proto\x1a\x15gateway/options.proto\x1a\x1fgoogle/api/field_behavior.proto\x1a\x19google/api/resource.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"\xd1\x03\n\tRatelimit\x12\x13\n\x04name\x18\x01 \x01(\tB\x05\xe2\x41\x02\x03\x05\x12\x31\n\ndeployment\x18\x02 \x01(\tB\x1d\xe2\x41\x02\x02\x05\xbaH\x04r\x02\x10\x01\xc2\xf1\x04\ridx_ratelimit\x12\'\n\x07\x61\x63\x63ount\x18\x03 \x01(\tB\x16\xe2\x41\x02\x01\x05\xc2\xf1\x04\ridx_ratelimit\x12-\n\x06metric\x18\x04 \x01(\tB\x1d\xe2\x41\x02\x02\x05\xbaH\x04r\x02\x10\x01\xc2\xf1\x04\ridx_ratelimit\x12\x1d\n\x05limit\x18\x05 \x01(\x02\x42\x0e\xe2\x41\x01\x01\xbaH\x07\n\x05-\x00\x00\x80\xbf\x12\x1b\n\ris_soft_limit\x18\x06 \x01(\x08\x42\x04\xe2\x41\x01\x01\x12#\n\x0breservation\x18\x07 \x01(\x02\x42\x0e\xe2\x41\x01\x01\xbaH\x07\n\x05-\x00\x00\x80\xbf\x12\x36\n\x0b\x63reate_time\x18\x08 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x05\xe2\x41\x02\x03\x05\x12\x35\n\x0bupdate_time\x18\t \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x04\xe2\x41\x01\x03:T\xea\x41M\n\x1a\x61pi.fireworks.ai/Ratelimit\x12\x18ratelimits/{RatelimitId}*\nratelimits2\tratelimit\x82\xf1\x04\x00\x42\x43ZAgithub.com/fw-ai/fireworks/control_plane/protos/generated/gatewayb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'gateway.ratelimit_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'ZAgithub.com/fw-ai/fireworks/control_plane/protos/generated/gateway'
  _globals['_RATELIMIT'].fields_by_name['name']._loaded_options = None
  _globals['_RATELIMIT'].fields_by_name['name']._serialized_options = b'\342A\002\003\005'
  _globals['_RATELIMIT'].fields_by_name['deployment']._loaded_options = None
  _globals['_RATELIMIT'].fields_by_name['deployment']._serialized_options = b'\342A\002\002\005\272H\004r\002\020\001\302\361\004\ridx_ratelimit'
  _globals['_RATELIMIT'].fields_by_name['account']._loaded_options = None
  _globals['_RATELIMIT'].fields_by_name['account']._serialized_options = b'\342A\002\001\005\302\361\004\ridx_ratelimit'
  _globals['_RATELIMIT'].fields_by_name['metric']._loaded_options = None
  _globals['_RATELIMIT'].fields_by_name['metric']._serialized_options = b'\342A\002\002\005\272H\004r\002\020\001\302\361\004\ridx_ratelimit'
  _globals['_RATELIMIT'].fields_by_name['limit']._loaded_options = None
  _globals['_RATELIMIT'].fields_by_name['limit']._serialized_options = b'\342A\001\001\272H\007\n\005-\000\000\200\277'
  _globals['_RATELIMIT'].fields_by_name['is_soft_limit']._loaded_options = None
  _globals['_RATELIMIT'].fields_by_name['is_soft_limit']._serialized_options = b'\342A\001\001'
  _globals['_RATELIMIT'].fields_by_name['reservation']._loaded_options = None
  _globals['_RATELIMIT'].fields_by_name['reservation']._serialized_options = b'\342A\001\001\272H\007\n\005-\000\000\200\277'
  _globals['_RATELIMIT'].fields_by_name['create_time']._loaded_options = None
  _globals['_RATELIMIT'].fields_by_name['create_time']._serialized_options = b'\342A\002\003\005'
  _globals['_RATELIMIT'].fields_by_name['update_time']._loaded_options = None
  _globals['_RATELIMIT'].fields_by_name['update_time']._serialized_options = b'\342A\001\003'
  _globals['_RATELIMIT']._loaded_options = None
  _globals['_RATELIMIT']._serialized_options = b'\352AM\n\032api.fireworks.ai/Ratelimit\022\030ratelimits/{RatelimitId}*\nratelimits2\tratelimit\202\361\004\000'
  _globals['_RATELIMIT']._serialized_start=182
  _globals['_RATELIMIT']._serialized_end=647
# @@protoc_insertion_point(module_scope)



import functools
from fireworks.control_plane.generated.protos.gateway import GatewayStub


class GatewayWrapper(GatewayStub):
    def __init__(self, *args, account_id: str, **kwargs):
        super().__init__(*args, **kwargs)

        if not account_id:
            raise ValueError("account_id must be provided and not")

        self.parent = f"accounts/{account_id}"

        # go through every method on self and wrap it
        for method_name in dir(self):
            if method_name.startswith("_"):
                continue
            method = getattr(self, method_name)
            if callable(method):
                setattr(self, method_name, self.wrap_method(method))

    def wrap_method(self, method):
        @functools.wraps(method)
        async def wrapper(*args, **kwargs):
            # if args[0] has attribute "parent"
            if hasattr(args[0], "parent"):
                args[0].parent = self.parent
            return await method(*args, **kwargs)

        return wrapper



from . import options_pb2 as _options_pb2
from . import status_pb2 as _status_pb2
from ..google.api import field_behavior_pb2 as _field_behavior_pb2
from ..google.api import resource_pb2 as _resource_pb2
from google.protobuf import field_mask_pb2 as _field_mask_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class PeftMergeJob(_message.Message):
    __slots__ = ("name", "display_name", "create_time", "created_by", "state", "status", "peft_model", "merged_model", "update_time")
    NAME_FIELD_NUMBER: _ClassVar[int]
    DISPLAY_NAME_FIELD_NUMBER: _ClassVar[int]
    CREATE_TIME_FIELD_NUMBER: _ClassVar[int]
    CREATED_BY_FIELD_NUMBER: _ClassVar[int]
    STATE_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    PEFT_MODEL_FIELD_NUMBER: _ClassVar[int]
    MERGED_MODEL_FIELD_NUMBER: _ClassVar[int]
    UPDATE_TIME_FIELD_NUMBER: _ClassVar[int]
    name: str
    display_name: str
    create_time: _timestamp_pb2.Timestamp
    created_by: str
    state: _status_pb2.JobState
    status: _status_pb2.Status
    peft_model: str
    merged_model: str
    update_time: _timestamp_pb2.Timestamp
    def __init__(self, name: _Optional[str] = ..., display_name: _Optional[str] = ..., create_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., created_by: _Optional[str] = ..., state: _Optional[_Union[_status_pb2.JobState, str]] = ..., status: _Optional[_Union[_status_pb2.Status, _Mapping]] = ..., peft_model: _Optional[str] = ..., merged_model: _Optional[str] = ..., update_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...) -> None: ...

class GetPeftMergeJobRequest(_message.Message):
    __slots__ = ("name", "read_mask")
    NAME_FIELD_NUMBER: _ClassVar[int]
    READ_MASK_FIELD_NUMBER: _ClassVar[int]
    name: str
    read_mask: _field_mask_pb2.FieldMask
    def __init__(self, name: _Optional[str] = ..., read_mask: _Optional[_Union[_field_mask_pb2.FieldMask, _Mapping]] = ...) -> None: ...

class CreatePeftMergeJobRequest(_message.Message):
    __slots__ = ("parent", "peft_merge_job")
    PARENT_FIELD_NUMBER: _ClassVar[int]
    PEFT_MERGE_JOB_FIELD_NUMBER: _ClassVar[int]
    parent: str
    peft_merge_job: PeftMergeJob
    def __init__(self, parent: _Optional[str] = ..., peft_merge_job: _Optional[_Union[PeftMergeJob, _Mapping]] = ...) -> None: ...

class ListPeftMergeJobsRequest(_message.Message):
    __slots__ = ("parent", "page_size", "page_token", "filter", "order_by", "read_mask")
    PARENT_FIELD_NUMBER: _ClassVar[int]
    PAGE_SIZE_FIELD_NUMBER: _ClassVar[int]
    PAGE_TOKEN_FIELD_NUMBER: _ClassVar[int]
    FILTER_FIELD_NUMBER: _ClassVar[int]
    ORDER_BY_FIELD_NUMBER: _ClassVar[int]
    READ_MASK_FIELD_NUMBER: _ClassVar[int]
    parent: str
    page_size: int
    page_token: str
    filter: str
    order_by: str
    read_mask: _field_mask_pb2.FieldMask
    def __init__(self, parent: _Optional[str] = ..., page_size: _Optional[int] = ..., page_token: _Optional[str] = ..., filter: _Optional[str] = ..., order_by: _Optional[str] = ..., read_mask: _Optional[_Union[_field_mask_pb2.FieldMask, _Mapping]] = ...) -> None: ...

class ListPeftMergeJobsResponse(_message.Message):
    __slots__ = ("peft_merge_jobs", "next_page_token", "total_size")
    PEFT_MERGE_JOBS_FIELD_NUMBER: _ClassVar[int]
    NEXT_PAGE_TOKEN_FIELD_NUMBER: _ClassVar[int]
    TOTAL_SIZE_FIELD_NUMBER: _ClassVar[int]
    peft_merge_jobs: _containers.RepeatedCompositeFieldContainer[PeftMergeJob]
    next_page_token: str
    total_size: int
    def __init__(self, peft_merge_jobs: _Optional[_Iterable[_Union[PeftMergeJob, _Mapping]]] = ..., next_page_token: _Optional[str] = ..., total_size: _Optional[int] = ...) -> None: ...

class DeletePeftMergeJobRequest(_message.Message):
    __slots__ = ("name",)
    NAME_FIELD_NUMBER: _ClassVar[int]
    name: str
    def __init__(self, name: _Optional[str] = ...) -> None: ...



from ..buf.validate import validate_pb2 as _validate_pb2
from . import options_pb2 as _options_pb2
from ..google.api import field_behavior_pb2 as _field_behavior_pb2
from ..google.api import resource_pb2 as _resource_pb2
from google.protobuf import field_mask_pb2 as _field_mask_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class IdentityProvider(_message.Message):
    __slots__ = ("name", "display_name", "create_time", "update_time", "saml_config", "oidc_config")
    NAME_FIELD_NUMBER: _ClassVar[int]
    DISPLAY_NAME_FIELD_NUMBER: _ClassVar[int]
    CREATE_TIME_FIELD_NUMBER: _ClassVar[int]
    UPDATE_TIME_FIELD_NUMBER: _ClassVar[int]
    SAML_CONFIG_FIELD_NUMBER: _ClassVar[int]
    OIDC_CONFIG_FIELD_NUMBER: _ClassVar[int]
    name: str
    display_name: str
    create_time: _timestamp_pb2.Timestamp
    update_time: _timestamp_pb2.Timestamp
    saml_config: SamlConfig
    oidc_config: OidcConfig
    def __init__(self, name: _Optional[str] = ..., display_name: _Optional[str] = ..., create_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., update_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., saml_config: _Optional[_Union[SamlConfig, _Mapping]] = ..., oidc_config: _Optional[_Union[OidcConfig, _Mapping]] = ...) -> None: ...

class SamlConfig(_message.Message):
    __slots__ = ("metadata_url",)
    METADATA_URL_FIELD_NUMBER: _ClassVar[int]
    metadata_url: str
    def __init__(self, metadata_url: _Optional[str] = ...) -> None: ...

class OidcConfig(_message.Message):
    __slots__ = ("issuer_url", "client_id", "client_secret")
    ISSUER_URL_FIELD_NUMBER: _ClassVar[int]
    CLIENT_ID_FIELD_NUMBER: _ClassVar[int]
    CLIENT_SECRET_FIELD_NUMBER: _ClassVar[int]
    issuer_url: str
    client_id: str
    client_secret: str
    def __init__(self, issuer_url: _Optional[str] = ..., client_id: _Optional[str] = ..., client_secret: _Optional[str] = ...) -> None: ...

class CreateIdentityProviderRequest(_message.Message):
    __slots__ = ("parent", "identity_provider")
    PARENT_FIELD_NUMBER: _ClassVar[int]
    IDENTITY_PROVIDER_FIELD_NUMBER: _ClassVar[int]
    parent: str
    identity_provider: IdentityProvider
    def __init__(self, parent: _Optional[str] = ..., identity_provider: _Optional[_Union[IdentityProvider, _Mapping]] = ...) -> None: ...

class GetIdentityProviderRequest(_message.Message):
    __slots__ = ("name",)
    NAME_FIELD_NUMBER: _ClassVar[int]
    name: str
    def __init__(self, name: _Optional[str] = ...) -> None: ...

class ListIdentityProvidersRequest(_message.Message):
    __slots__ = ("parent", "page_size", "page_token", "filter")
    PARENT_FIELD_NUMBER: _ClassVar[int]
    PAGE_SIZE_FIELD_NUMBER: _ClassVar[int]
    PAGE_TOKEN_FIELD_NUMBER: _ClassVar[int]
    FILTER_FIELD_NUMBER: _ClassVar[int]
    parent: str
    page_size: int
    page_token: str
    filter: str
    def __init__(self, parent: _Optional[str] = ..., page_size: _Optional[int] = ..., page_token: _Optional[str] = ..., filter: _Optional[str] = ...) -> None: ...

class ListIdentityProvidersResponse(_message.Message):
    __slots__ = ("identity_providers", "next_page_token", "total_size")
    IDENTITY_PROVIDERS_FIELD_NUMBER: _ClassVar[int]
    NEXT_PAGE_TOKEN_FIELD_NUMBER: _ClassVar[int]
    TOTAL_SIZE_FIELD_NUMBER: _ClassVar[int]
    identity_providers: _containers.RepeatedCompositeFieldContainer[IdentityProvider]
    next_page_token: str
    total_size: int
    def __init__(self, identity_providers: _Optional[_Iterable[_Union[IdentityProvider, _Mapping]]] = ..., next_page_token: _Optional[str] = ..., total_size: _Optional[int] = ...) -> None: ...

class UpdateIdentityProviderRequest(_message.Message):
    __slots__ = ("identity_provider", "update_mask")
    IDENTITY_PROVIDER_FIELD_NUMBER: _ClassVar[int]
    UPDATE_MASK_FIELD_NUMBER: _ClassVar[int]
    identity_provider: IdentityProvider
    update_mask: _field_mask_pb2.FieldMask
    def __init__(self, identity_provider: _Optional[_Union[IdentityProvider, _Mapping]] = ..., update_mask: _Optional[_Union[_field_mask_pb2.FieldMask, _Mapping]] = ...) -> None: ...

class DeleteIdentityProviderRequest(_message.Message):
    __slots__ = ("name",)
    NAME_FIELD_NUMBER: _ClassVar[int]
    name: str
    def __init__(self, name: _Optional[str] = ...) -> None: ...



from typing import (
    AsyncGenerator,
    Generator,
    Iterable,
    List,
    Literal,
    Optional,
    Union,
    overload,
    TYPE_CHECKING,
)

from fireworks.llm._types import ResponseFormat
from openai import AsyncOpenAI, NotGiven, NOT_GIVEN
from openai.types.responses import Response as OpenAIResponse
from openai.types.responses import ResponseStreamEvent
from openai.types.responses import ResponseCreateParams
from openai.types.responses import ResponseInputParam
from openai.types.responses import ResponseTextConfigParam
from openai.types.responses import ResponseIncludable
from openai.types.responses import ToolParam
from openai.types.shared_params.metadata import Metadata
from openai.types.shared_params.reasoning import Reasoning
from openai.types.shared_params.responses_model import ResponsesModel
from openai.types.responses.response_create_params import ToolChoice
from openai._streaming import Stream, AsyncStream

from fireworks._literals import ReasoningEffort
from fireworks.llm.base_completion import BaseCompletion

# Type checking imports to avoid circular imports
if TYPE_CHECKING:
    from fireworks.llm.llm import LLM


class Responses(BaseCompletion):
    def __init__(self, llm: "LLM"):
        super().__init__(llm)
        # Import OpenAI client here to avoid circular imports
        from openai import OpenAI

        self._openai_client = OpenAI(
            api_key=llm._client.api_key,
            base_url=str(llm._client.base_url),
            max_retries=llm.max_retries,
        )
        self._openai_client_async = AsyncOpenAI(
            api_key=llm._client.api_key,
            base_url=str(llm._client.base_url),
            max_retries=llm.max_retries,
        )

    @overload
    def create(
        self,
        *,
        input: Union[str, ResponseInputParam],
        include: Optional[List[ResponseIncludable]] | NotGiven = NOT_GIVEN,
        instructions: Optional[str] | NotGiven = NOT_GIVEN,
        max_output_tokens: Optional[int] | NotGiven = NOT_GIVEN,
        metadata: Optional[Metadata] | NotGiven = NOT_GIVEN,
        parallel_tool_calls: Optional[bool] | NotGiven = NOT_GIVEN,
        previous_response_id: Optional[str] | NotGiven = NOT_GIVEN,
        reasoning: Optional[Reasoning] | NotGiven = NOT_GIVEN,
        service_tier: Optional[Literal["auto", "default", "flex"]] | NotGiven = NOT_GIVEN,
        store: Optional[bool] | NotGiven = NOT_GIVEN,
        stream: Optional[Literal[False]] | NotGiven = NOT_GIVEN,
        temperature: Optional[float] | NotGiven = NOT_GIVEN,
        text: ResponseTextConfigParam | NotGiven = NOT_GIVEN,
        tool_choice: ToolChoice | NotGiven = NOT_GIVEN,
        tools: Iterable[ToolParam] | NotGiven = NOT_GIVEN,
        top_p: Optional[float] | NotGiven = NOT_GIVEN,
        truncation: Optional[Literal["auto", "disabled"]] | NotGiven = NOT_GIVEN,
        user: str | NotGiven = NOT_GIVEN,
        extra_headers=None,
        **kwargs,
    ) -> OpenAIResponse:
        pass

    @overload
    def create(
        self,
        *,
        input: Union[str, ResponseInputParam],
        stream: Literal[True],
        include: Optional[List[ResponseIncludable]] | NotGiven = NOT_GIVEN,
        instructions: Optional[str] | NotGiven = NOT_GIVEN,
        max_output_tokens: Optional[int] | NotGiven = NOT_GIVEN,
        metadata: Optional[Metadata] | NotGiven = NOT_GIVEN,
        parallel_tool_calls: Optional[bool] | NotGiven = NOT_GIVEN,
        previous_response_id: Optional[str] | NotGiven = NOT_GIVEN,
        reasoning: Optional[Reasoning] | NotGiven = NOT_GIVEN,
        service_tier: Optional[Literal["auto", "default", "flex"]] | NotGiven = NOT_GIVEN,
        store: Optional[bool] | NotGiven = NOT_GIVEN,
        temperature: Optional[float] | NotGiven = NOT_GIVEN,
        text: ResponseTextConfigParam | NotGiven = NOT_GIVEN,
        tool_choice: ToolChoice | NotGiven = NOT_GIVEN,
        tools: Iterable[ToolParam] | NotGiven = NOT_GIVEN,
        top_p: Optional[float] | NotGiven = NOT_GIVEN,
        truncation: Optional[Literal["auto", "disabled"]] | NotGiven = NOT_GIVEN,
        user: str | NotGiven = NOT_GIVEN,
        extra_headers=None,
        **kwargs,
    ) -> Stream[ResponseStreamEvent]:
        pass

    def create(
        self,
        *,
        input: Union[str, ResponseInputParam],
        include: Optional[List[ResponseIncludable]] | NotGiven = NOT_GIVEN,
        instructions: Optional[str] | NotGiven = NOT_GIVEN,
        max_output_tokens: Optional[int] | NotGiven = NOT_GIVEN,
        metadata: Optional[Metadata] | NotGiven = NOT_GIVEN,
        parallel_tool_calls: Optional[bool] | NotGiven = NOT_GIVEN,
        previous_response_id: Optional[str] | NotGiven = NOT_GIVEN,
        reasoning: Optional[Reasoning] | NotGiven = NOT_GIVEN,
        service_tier: Optional[Literal["auto", "default", "flex"]] | NotGiven = NOT_GIVEN,
        store: Optional[bool] | NotGiven = NOT_GIVEN,
        stream: Optional[bool] | NotGiven = NOT_GIVEN,
        temperature: Optional[float] | NotGiven = NOT_GIVEN,
        text: ResponseTextConfigParam | NotGiven = NOT_GIVEN,
        tool_choice: ToolChoice | NotGiven = NOT_GIVEN,
        tools: Iterable[ToolParam] | NotGiven = NOT_GIVEN,
        top_p: Optional[float] | NotGiven = NOT_GIVEN,
        truncation: Optional[Literal["auto", "disabled"]] | NotGiven = NOT_GIVEN,
        user: str | NotGiven = NOT_GIVEN,
        extra_headers=None,
        **kwargs,
    ) -> Union[OpenAIResponse, Stream[ResponseStreamEvent]]:
        model_id = self._create_setup()
        params = self._build_request_params(
            model_id,
            input,
            stream,
            include,
            instructions,
            max_output_tokens,
            metadata,
            parallel_tool_calls,
            previous_response_id,
            reasoning,
            service_tier,
            store,
            temperature,
            text,
            tool_choice,
            tools,
            top_p,
            truncation,
            user,
            extra_headers,
            **kwargs,
        )

        return self._execute_with_retry(params, "responses")

    @overload
    async def acreate(
        self,
        *,
        input: Union[str, ResponseInputParam],
        include: Optional[List[ResponseIncludable]] | NotGiven = NOT_GIVEN,
        instructions: Optional[str] | NotGiven = NOT_GIVEN,
        max_output_tokens: Optional[int] | NotGiven = NOT_GIVEN,
        metadata: Optional[Metadata] | NotGiven = NOT_GIVEN,
        parallel_tool_calls: Optional[bool] | NotGiven = NOT_GIVEN,
        previous_response_id: Optional[str] | NotGiven = NOT_GIVEN,
        reasoning: Optional[Reasoning] | NotGiven = NOT_GIVEN,
        service_tier: Optional[Literal["auto", "default", "flex"]] | NotGiven = NOT_GIVEN,
        store: Optional[bool] | NotGiven = NOT_GIVEN,
        stream: Optional[Literal[False]] | NotGiven = NOT_GIVEN,
        temperature: Optional[float] | NotGiven = NOT_GIVEN,
        text: ResponseTextConfigParam | NotGiven = NOT_GIVEN,
        tool_choice: ToolChoice | NotGiven = NOT_GIVEN,
        tools: Iterable[ToolParam] | NotGiven = NOT_GIVEN,
        top_p: Optional[float] | NotGiven = NOT_GIVEN,
        truncation: Optional[Literal["auto", "disabled"]] | NotGiven = NOT_GIVEN,
        user: str | NotGiven = NOT_GIVEN,
        extra_headers=None,
        **kwargs,
    ) -> OpenAIResponse:
        pass

    @overload
    async def acreate(
        self,
        *,
        input: Union[str, ResponseInputParam],
        stream: Literal[True],
        include: Optional[List[ResponseIncludable]] | NotGiven = NOT_GIVEN,
        instructions: Optional[str] | NotGiven = NOT_GIVEN,
        max_output_tokens: Optional[int] | NotGiven = NOT_GIVEN,
        metadata: Optional[Metadata] | NotGiven = NOT_GIVEN,
        parallel_tool_calls: Optional[bool] | NotGiven = NOT_GIVEN,
        previous_response_id: Optional[str] | NotGiven = NOT_GIVEN,
        reasoning: Optional[Reasoning] | NotGiven = NOT_GIVEN,
        service_tier: Optional[Literal["auto", "default", "flex"]] | NotGiven = NOT_GIVEN,
        store: Optional[bool] | NotGiven = NOT_GIVEN,
        temperature: Optional[float] | NotGiven = NOT_GIVEN,
        text: ResponseTextConfigParam | NotGiven = NOT_GIVEN,
        tool_choice: ToolChoice | NotGiven = NOT_GIVEN,
        tools: Iterable[ToolParam] | NotGiven = NOT_GIVEN,
        top_p: Optional[float] | NotGiven = NOT_GIVEN,
        truncation: Optional[Literal["auto", "disabled"]] | NotGiven = NOT_GIVEN,
        user: str | NotGiven = NOT_GIVEN,
        extra_headers=None,
        **kwargs,
    ) -> AsyncStream[ResponseStreamEvent]:
        pass

    async def acreate(
        self,
        *,
        input: Union[str, ResponseInputParam],
        include: Optional[List[ResponseIncludable]] | NotGiven = NOT_GIVEN,
        instructions: Optional[str] | NotGiven = NOT_GIVEN,
        max_output_tokens: Optional[int] | NotGiven = NOT_GIVEN,
        metadata: Optional[Metadata] | NotGiven = NOT_GIVEN,
        parallel_tool_calls: Optional[bool] | NotGiven = NOT_GIVEN,
        previous_response_id: Optional[str] | NotGiven = NOT_GIVEN,
        reasoning: Optional[Reasoning] | NotGiven = NOT_GIVEN,
        service_tier: Optional[Literal["auto", "default", "flex"]] | NotGiven = NOT_GIVEN,
        store: Optional[bool] | NotGiven = NOT_GIVEN,
        stream: Optional[bool] | NotGiven = NOT_GIVEN,
        temperature: Optional[float] | NotGiven = NOT_GIVEN,
        text: ResponseTextConfigParam | NotGiven = NOT_GIVEN,
        tool_choice: ToolChoice | NotGiven = NOT_GIVEN,
        tools: Iterable[ToolParam] | NotGiven = NOT_GIVEN,
        top_p: Optional[float] | NotGiven = NOT_GIVEN,
        truncation: Optional[Literal["auto", "disabled"]] | NotGiven = NOT_GIVEN,
        user: str | NotGiven = NOT_GIVEN,
        extra_headers=None,
        **kwargs,
    ) -> Union[OpenAIResponse, AsyncStream[ResponseStreamEvent]]:
        model_id = self._create_setup()
        params = self._build_request_params(
            model_id,
            input,
            stream,
            include,
            instructions,
            max_output_tokens,
            metadata,
            parallel_tool_calls,
            previous_response_id,
            reasoning,
            service_tier,
            store,
            temperature,
            text,
            tool_choice,
            tools,
            top_p,
            truncation,
            user,
            extra_headers,
            **kwargs,
        )

        return await self._execute_with_retry_async(params, "responses")

    def _execute_with_retry(self, params: dict, operation_name: str):
        metric = self._llm.enable_metrics and not params.get("stream", False)
        return self._retry_with_backoff(
            self._openai_client.responses.create, **params, operation_name=operation_name, metric=metric
        )

    async def _execute_with_retry_async(self, params: dict, operation_name: str):
        metric = self._llm.enable_metrics and not params.get("stream", False)
        return await self._retry_with_backoff_async(
            self._openai_client_async.responses.create,
            **params,
            operation_name=operation_name,
            metric=metric,
        )

    def _build_request_params(
        self,
        model_id: str,
        input: Union[str, ResponseInputParam],
        stream: Optional[bool] | NotGiven = NOT_GIVEN,
        include: Optional[List[ResponseIncludable]] | NotGiven = NOT_GIVEN,
        instructions: Optional[str] | NotGiven = NOT_GIVEN,
        max_output_tokens: Optional[int] | NotGiven = NOT_GIVEN,
        metadata: Optional[Metadata] | NotGiven = NOT_GIVEN,
        parallel_tool_calls: Optional[bool] | NotGiven = NOT_GIVEN,
        previous_response_id: Optional[str] | NotGiven = NOT_GIVEN,
        reasoning: Optional[Reasoning] | NotGiven = NOT_GIVEN,
        service_tier: Optional[Literal["auto", "default", "flex"]] | NotGiven = NOT_GIVEN,
        store: Optional[bool] | NotGiven = NOT_GIVEN,
        temperature: Optional[float] | NotGiven = NOT_GIVEN,
        text: ResponseTextConfigParam | NotGiven = NOT_GIVEN,
        tool_choice: ToolChoice | NotGiven = NOT_GIVEN,
        tools: Iterable[ToolParam] | NotGiven = NOT_GIVEN,
        top_p: Optional[float] | NotGiven = NOT_GIVEN,
        truncation: Optional[Literal["auto", "disabled"]] | NotGiven = NOT_GIVEN,
        user: str | NotGiven = NOT_GIVEN,
        extra_headers=None,
        **kwargs,
    ) -> dict:
        """Build request parameters for responses calls."""
        # Build common parameters using base class method
        params = self._build_common_request_params(
            model_id=model_id,
            stream=stream,
            max_tokens=max_output_tokens,
            temperature=temperature,
            top_p=top_p,
            extra_headers=extra_headers,
            **kwargs,
        )

        # Add responses-specific parameters
        params["input"] = input
        params["model"] = model_id

        # Add optional responses-specific parameters if not NOT_GIVEN
        optional_params = {
            "include": include,
            "instructions": instructions,
            "metadata": metadata,
            "parallel_tool_calls": parallel_tool_calls,
            "previous_response_id": previous_response_id,
            "reasoning": reasoning,
            "service_tier": service_tier,
            "store": store,
            "text": text,
            "tool_choice": tool_choice,
            "tools": tools,
            "truncation": truncation,
            "user": user,
        }

        for key, value in optional_params.items():
            if value is not NOT_GIVEN:
                params[key] = value

        return params



import os


FIREWORKS_API_BASE_URL = os.environ.get("FIREWORKS_API_BASE_URL", "https://api.fireworks.ai")
FIREWORKS_GATEWAY_ADDR = os.environ.get("FIREWORKS_GATEWAY_ADDR", "gateway.fireworks.ai:443")



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: google/type/money.proto
# Protobuf Python Version: 5.29.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    0,
    '',
    'google/type/money.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x17google/type/money.proto\x12\x0bgoogle.type\"<\n\x05Money\x12\x15\n\rcurrency_code\x18\x01 \x01(\t\x12\r\n\x05units\x18\x02 \x01(\x03\x12\r\n\x05nanos\x18\x03 \x01(\x05\x42`\n\x0f\x63om.google.typeB\nMoneyProtoP\x01Z6google.golang.org/genproto/googleapis/type/money;money\xf8\x01\x01\xa2\x02\x03GTPb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'google.type.money_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\017com.google.typeB\nMoneyProtoP\001Z6google.golang.org/genproto/googleapis/type/money;money\370\001\001\242\002\003GTP'
  _globals['_MONEY']._serialized_start=40
  _globals['_MONEY']._serialized_end=100
# @@protoc_insertion_point(module_scope)



from datetime import datetime
import io
import json
import logging
import os
from functools import cache as sync_cache
from fireworks._util import (
    make_valid_resource_name,
)
from fireworks.evaluation_job.evaluation_job import EvaluationJob
from fireworks.evaluator.evaluator import Evaluator
import httpx
import atexit
from typing import Literal, Optional, Union, BinaryIO, Callable, overload
from fireworks.control_plane.generated.protos.gateway import (
    CreateDatasetRequest,
    ListDatasetsRequest,
    Dataset as DatasetProto,
)
from fireworks.control_plane.generated.protos_grpcio.gateway.dataset_pb2 import (
    Dataset as SyncDataset,
    ListDatasetsRequest as SyncListDatasetsRequest,
    CreateDatasetRequest as SyncCreateDatasetRequest,
)
from fireworks.control_plane.generated.protos_grpcio.gateway.evaluation_job_pb2 import (
    CreateEvaluationJobRequest as SyncCreateEvaluationJobRequest,
    GetEvaluationJobRequest as SyncGetEvaluationJobRequest,
    EvaluationJob as SyncEvaluationJob,
)

import mmh3

from fireworks.gateway import Gateway

logger = logging.getLogger(__name__)
handler = logging.StreamHandler()
formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
handler.setFormatter(formatter)
logger.addHandler(handler)
logger.propagate = False  # Prevent duplicate logs

if os.environ.get("FIREWORKS_SDK_DEBUG"):
    logger.setLevel(logging.DEBUG)


class Dataset:
    def __init__(
        self,
        api_key: Optional[str] = None,
        path: Optional[str] = None,
        data: Optional[Union[str, list]] = None,
        id: Optional[str] = None,
        _internal=False,
    ):
        """
        A "smart" Dataset class that helps developers upload datasets and
        fine-tune on Fireworks. It has the following features:

        1. Works nicely with the `fireworks.llm.LLM` class.
        2. Automatically names dataset based on the contents of the dataset and the filename of the file if provided.
        3. Automatically uploads dataset to Fireworks if it doesn't already exist.

        To instantiate a Dataset, use the `from_*` class methods:
        - `from_dict(data: list)`
        - `from_file(path: str)`
        - `from_string(data: str)`
        - `from_id(id: str)`

        Args:
            path: Path to the local directory containing the dataset.
            data: Data to be uploaded to the dataset. Can be a string in JSONL format with OpenAI chat completion
                  message compatible JSON, or a list of OpenAI chat completion message compatible objects.
            id: The ID of an existing dataset on Fireworks.
        TODO: support various remote paths to cloud storage.
        """
        if not _internal:
            raise ValueError(
                "Dataset is not meant to be instantiated directly, use Dataset.from_dict() or Dataset.from_file() instead"
            )
        self._data: Optional[str] = None
        self._path: Optional[str] = None
        self._id: Optional[str] = id
        self._gateway = Gateway(api_key=api_key)
        self._file_stream: Optional[BinaryIO] = None
        atexit.register(self._gateway._channel.close)

        # Ensure exactly one of path, data, or id is provided
        provided_params = [path is not None, data is not None, id is not None]
        if sum(provided_params) == 0:
            raise ValueError("Must provide exactly one of: path, data, or id")
        if sum(provided_params) > 1:
            raise ValueError("Must provide exactly one of: path, data, or id (cannot provide multiple)")

        if path and not path.endswith(".jsonl"):
            raise ValueError("File must be a JSONL file")

        if path:
            self._path = path
        elif isinstance(data, list):
            # Convert list to newline delimited JSON string
            self._data = "\n".join(json.dumps(item) for item in data)
        elif isinstance(data, str):
            self._data = data
        elif id:
            self._id = id

    @classmethod
    def from_string(cls, data: str):
        return cls(data=data, _internal=True)

    @classmethod
    def from_list(cls, data: list):
        return cls(data=data, _internal=True)

    @classmethod
    def from_id(cls, id: str):
        return cls(id=id, _internal=True)

    @classmethod
    def from_file(cls, path: str):
        if not path.endswith(".jsonl"):
            raise ValueError("File must be a JSONL file")
        return cls(path=path, _internal=True)

    def get(self) -> Optional[SyncDataset]:
        """
        Get this dataset from Fireworks by hash
        - If filename of dataset changes, it still matches the hash
        """
        request = SyncListDatasetsRequest(page_size=1000)
        datasets = self._gateway.list_datasets_sync(request)
        for dataset in datasets:
            if dataset.name.endswith(self.id):
                logger.debug(f"Found dataset with matching hash: {dataset.name}")
                return dataset
        logger.debug(f"No dataset found with matching hash: {hash(self)}")
        return None

    @property
    def url(self) -> str:
        return f"https://app.fireworks.ai/dashboard/datasets/{self.id}"

    def sync(self):
        """
        Upload this dataset to Fireworks if it doesn't already exist.
        """
        # check if dataset exists by hash
        dataset = self.get()
        if dataset:
            logger.debug(f"Dataset already exists: {dataset.name}, no need to upload")
            return
        logger.debug(f"No dataset found with matching hash: {hash(self)}, creating new dataset")
        dataset = SyncDataset(
            format=self._detect_dataset_format(),
            example_count=min(self._line_count(), 50_000_000),
        )
        # upload dataset since it doesn't exist
        logger.debug(f"Creating dataset: {self.id}")
        request = SyncCreateDatasetRequest(dataset=dataset, dataset_id=self.id)
        dataset = self._gateway.create_dataset_sync(request)
        logger.debug(f"Dataset created: {dataset.name}")
        logger.debug(f"Uploading dataset: {self.id}")
        filename_to_size = {self.filename(): self.file_size()}
        signed_urls = self._gateway.get_dataset_upload_endpoint_sync(self.id, filename_to_size)
        self._upload_file_using_signed_url(signed_urls[self.filename()])
        logger.debug(f"Dataset uploaded: {self.id}")
        self._gateway.validate_dataset_sync(self.id)
        logger.debug(f"Dataset validated: {self.id}")

    def _upload_file_using_signed_url(self, signed_url: str) -> None:
        """
        Upload a file to a signed URL using async streaming to avoid loading the entire file into memory.

        Args:
            signed_url: The signed URL to upload the file to.
        """
        logger.debug(f"Uploading file to signed URL: {signed_url}")

        file = self._get_stream()
        try:
            # Get file size for content length
            file.seek(0, os.SEEK_END)
            size = file.tell()
            file.seek(0)

            # Prepare the HTTP request
            headers = {
                "Content-Type": "application/octet-stream",
                "X-Goog-Content-Length-Range": f"{size},{size}",
            }

            # Upload the file with streaming
            with httpx.Client() as client:
                response = client.put(signed_url, content=file, headers=headers)
                if response.status_code != 200:
                    raise RuntimeError(f"Failed to upload file: {response.status_code} - {response.text}")

            logger.debug("File upload completed successfully")
        finally:
            file.close()

    def filename(self) -> str:
        """
        Returns the filename of the dataset if path is provided, otherwise returns "inmemory"
        to indicate this dataset was created from an in-memory data structure rather than a file.
        """
        if self._path:
            return os.path.basename(self._path)
        return "inmemory.jsonl"

    @property
    def id(self):
        """
        Generates an id for this dataset in the form of "dataset-{hash(self)}-{filename}"
        For datasets created from an existing ID, extracts the name from the ID.
        """
        if self._id:
            return self._id
        return f"dataset-{hash(self)}-{make_valid_resource_name(self.filename())}"

    @property
    @sync_cache
    def name(self):
        return self.construct_name(self._gateway.account_id(), self.id)

    @classmethod
    def construct_name(cls, account_id: str, id: str):
        if id.startswith("accounts/"):
            raise ValueError(f"ID cannot start with 'accounts/': {id}")
        return f"accounts/{account_id}/datasets/{id}"

    @property
    def stream(self) -> BinaryIO:
        """
        Returns a cached file-like object for the dataset content.
        For backward compatibility - prefer using get_stream() with a context manager.
        """
        if self._file_stream is not None and not self._file_stream.closed:
            self._file_stream.seek(0)
            return self._file_stream

        self._file_stream = self._get_stream()
        return self._file_stream

    def _line_count(self) -> int:
        """
        Returns the number of lines in the dataset
        """
        with self._get_stream() as stream:
            count = 0
            for _ in stream:
                count += 1
            return count

    def _detect_dataset_format(self) -> SyncDataset.Format:
        """
        Detects the format of the dataset by examining its content.

        Returns:
            A string representing the dataset format:
            - "CHAT" for chat completion format
            - "COMPLETION" for completion format
            - "FORMAT_UNSPECIFIED" if the format cannot be determined
        """

        try:
            with self._get_stream() as file:
                for line in file:
                    try:
                        data = json.loads(line.decode("utf-8"))

                        # Check for completion format (prompt + completion)
                        if "prompt" in data and "completion" in data:
                            return SyncDataset.Format.COMPLETION

                        # Check for chat format (messages array)
                        if "messages" in data and isinstance(data["messages"], list) and len(data["messages"]) > 0:
                            return SyncDataset.Format.CHAT

                        # If we reach here, the format doesn't match either completion or chat
                        return SyncDataset.Format.FORMAT_UNSPECIFIED
                    except json.JSONDecodeError:
                        return SyncDataset.Format.FORMAT_UNSPECIFIED

                # If we've read the entire file without determining a format, it's unspecified
                return SyncDataset.Format.FORMAT_UNSPECIFIED
        except Exception as e:
            logger.error(f"Error detecting dataset format: {e}")
            return SyncDataset.Format.FORMAT_UNSPECIFIED

    def _get_stream(self) -> BinaryIO:
        """
        Returns a fresh file-like object for the dataset content.
        For in-memory data, returns a BytesIO object.
        For file paths, returns a file object.
        This is intended to be used with a context manager to ensure proper cleanup.
        """
        if self._data:
            return io.BytesIO(self._data.encode("utf-8"))
        elif self._path:
            return open(self._path, "rb")
        elif self._id:
            response = self._gateway.get_dataset_download_endpoint_sync(self._id)
            if not response:
                raise ValueError(f'Dataset with id "{self._id}" does not exist')
            signed_url = next(iter(response.filename_to_signed_urls.values()))
            # download file to in-memory bytesio
            with httpx.Client() as client:
                response = client.get(signed_url)
                return io.BytesIO(response.content)
        else:
            raise ValueError("No data or path provided")

    def __iter__(self):
        """
        Make the dataset iterable, yielding parsed JSON objects from each line.

        Yields:
            dict: Parsed JSON object from each line in the dataset

        Example:
            for record in dataset:
                print(record)  # Each record is a parsed JSON dict
        """
        stream = self.stream  # Use the cached stream property
        stream.seek(0)  # Ensure we start from the beginning
        for line in stream:
            line_str = line.decode("utf-8").strip()
            if line_str:  # Skip empty lines
                try:
                    yield json.loads(line_str)
                except json.JSONDecodeError as e:
                    logger.warning(f"Failed to parse JSON line: {line_str[:100]}... Error: {e}")
                    continue

    def file_size(self) -> int:
        """
        Returns the size of the dataset in bytes without reading the entire file
        """
        # Use optimized path for files since we can get size without reading
        if self._path:
            return os.path.getsize(self._path)

        # For in-memory data, get the size from the stream
        with self._get_stream() as stream:
            stream.seek(0, io.SEEK_END)
            size = stream.tell()
            return size

    def __hash__(self) -> int:
        """
        Computes a hash of the dataset contents
        """
        # Read the entire file content for hashing
        content = self.read()
        # Ensure the hash is positive by using the unsigned value (& with MAXINT)
        return mmh3.hash(content) & 0x7FFFFFFF

    def delete(self):
        """
        Delete this dataset from Fireworks
        """
        # if dataset doesn't exist, don't delete
        dataset = self.get()
        if not dataset:
            logger.debug(f"Dataset does not exist: {self.id}, no need to delete")
            return
        self._gateway.delete_dataset_sync(self.id)

    def read(self, size: Optional[int] = None) -> bytes:
        """
        Read content from the dataset

        Args:
            size: Number of bytes to read, or None to read the entire file
                 (use cautiously with large files)

        Returns:
            The content as bytes
        """
        with self._get_stream() as stream:
            if size is not None:
                return stream.read(size)
            return stream.read()

    def preview_evaluator(self, reward_function: Callable, samples: Optional[int] = None):
        """
        Preview the evaluator for the dataset
        """
        if samples:
            new_dataset = self.head(samples, as_dataset=True)
            return new_dataset.preview_evaluator(reward_function)

        evaluator = Evaluator(gateway=self._gateway, reward_function=reward_function)
        evaluator.sync()
        return evaluator.preview(self)

    def create_evaluation_job(self, reward_function: Callable, samples: Optional[int] = None) -> EvaluationJob:
        """
        Create an evaluation job using a reward function for this dataset

        Args:
            reward_function: A callable decorated with @reward_function
            samples: Optional number of samples to evaluate (creates a subset dataset)

        Returns:
            EvaluationJob: The created evaluation job
        """
        # if samples is provided, create a new dataset with only those samples
        # and create an evaluation job on that dataset
        if samples:
            new_dataset = self.head(samples, as_dataset=True)
            return new_dataset.create_evaluation_job(reward_function)

        self.sync()

        # Create and sync the evaluator
        evaluator = Evaluator(gateway=self._gateway, reward_function=reward_function)
        evaluator.sync()

        # Create the evaluation job
        output_dataset_id = datetime.now().strftime("%Y-%m-%d-%H-%M-%S-%f")
        evaluation_job_proto = SyncEvaluationJob(
            evaluator=evaluator.name,
            input_dataset=self.name,
            output_dataset=self.construct_name(self._gateway.account_id(), output_dataset_id),
            display_name=output_dataset_id,
        )

        # create evaluation job with evaluator
        evaluation_job = EvaluationJob(
            gateway=self._gateway,
            evaluation_job=evaluation_job_proto,
        )
        evaluation_job.sync()
        return evaluation_job

    @overload
    def head(self, n: int = 5, as_dataset: Literal[False] = False) -> list: ...

    @overload
    def head(self, n: int = 5, as_dataset: Literal[True] = True) -> "Dataset": ...

    def head(self, n: int = 5, as_dataset: bool = False) -> Union[list, "Dataset"]:
        """
        Return the first n rows of the dataset.

        Args:
            n: Number of rows to return (default: 5)
            as_dataset: If True, return a Dataset object; if False, return a list (default: False)

        Returns:
            list or Dataset: List of dictionaries if as_dataset=False, Dataset object if as_dataset=True

        Example:
            dataset = Dataset.from_file("data.jsonl")
            first_5_rows = dataset.head(5)  # Returns list
            first_5_dataset = dataset.head(5, as_dataset=True)  # Returns Dataset
        """
        rows = []
        for i, row in enumerate(self):
            if i >= n:
                break
            rows.append(row)

        if as_dataset:
            return Dataset.from_list(rows)
        return rows

    def __getitem__(self, key):
        """
        Support slice notation and indexing for the dataset.

        Args:
            key: Either a slice object (e.g., [:5], [2:10], [::2]) or an integer index

        Returns:
            list: For slices, returns a list of dictionaries representing the sliced rows
            dict: For single index, returns a single dictionary representing that row

        Examples:
            dataset[:5]    # Returns first 5 rows (calls head(5))
            dataset[2:7]   # Returns rows 2-6
            dataset[10]    # Returns the 10th row (0-indexed)
        """
        if isinstance(key, slice):
            start = key.start or 0
            stop = key.stop
            step = key.step or 1

            if start == 0 and step == 1 and stop is not None:
                # Simple case: [:n] - can use head method for efficiency
                return self.head(stop)
            else:
                # More complex slicing - need to iterate through dataset
                rows = []
                for i, row in enumerate(self):
                    if stop is not None and i >= stop:
                        break
                    if i >= start and (step == 1 or (i - start) % step == 0):
                        rows.append(row)
                return rows
        else:
            # Handle single index access
            if key < 0:
                raise IndexError("Negative indexing not supported for Dataset")
            for i, row in enumerate(self):
                if i == key:
                    return row
            raise IndexError(f"Index {key} out of range")

    def __eq__(self, other):
        """
        Check if two datasets are equal.

        Args:
            other: The other dataset to compare with.

        Returns:
            bool: True if the datasets are equal, False otherwise
        """
        if not isinstance(other, Dataset):
            return False
        return self.id == other.id and self.file_size() == other.file_size()



"""
Evaluator module for managing Fireworks evaluators.

This module provides the Evaluator class which handles the creation and management
of evaluators on the Fireworks platform. Evaluators are created from reward functions
and can be reused across multiple evaluation jobs.

The Evaluator class follows the same pattern as EvaluationJob, providing:
- Object-oriented interface for evaluator management
- Automatic versioning of evaluators
- Reuse of identical evaluators to avoid duplication
- Integration with the Fireworks Gateway

Example usage:
    from fireworks import Evaluator
    from fireworks.gateway import Gateway

    gateway = Gateway()
    evaluator = Evaluator(gateway=gateway, reward_function=my_reward_function)
    evaluator.sync()  # Creates or finds existing evaluator
    print(f"Evaluator URL: {evaluator.url}")
"""

import ast
import inspect
import json
import os
import re
from datetime import datetime
from typing import Callable, List, Optional

from fireworks._util import (
    dedent_code,
    find_requirements_or_pyproject,
    get_requirements_from_pyproject_or_requirements_txt,
)
from fireworks.control_plane.generated.protos_grpcio.gateway.evaluator_pb2 import (
    Evaluator as SyncEvaluator,
    Criterion as SyncCriterion,
    CodeSnippets as SyncCodeSnippets,
    RollupSettings as SyncRollupSettings,
    CreateEvaluatorRequest as SyncCreateEvaluatorRequest,
    ListEvaluatorsRequest as SyncListEvaluatorsRequest,
    PreviewEvaluatorRequest as SyncPreviewEvaluatorRequest,
    PreviewEvaluatorResponse as SyncPreviewEvaluatorResponse,
)
from fireworks.gateway import Gateway
from fireworks._logger import logger


class Evaluator:
    VERSION_DELIMITER = "-v"

    def __init__(self, gateway: Gateway, reward_function: Callable):
        """
        An Evaluator class that manages the creation and retrieval of evaluators on Fireworks.

        Args:
            gateway: The Fireworks Gateway instance
            reward_function: A callable decorated with @reward_function
        """
        self._gateway = gateway
        self._reward_function = reward_function
        self._proto: Optional[SyncEvaluator] = None

        # Validate the reward function
        self._validate_reward_function()

    @property
    def _reward_function_name(self) -> str:
        """Get the name of the reward function"""
        return self._reward_function.__name__

    @property
    def _unwrapped_reward_function(self) -> Callable:
        """Unwrap the reward function completely and cache the result"""
        # unwrap until not unwrappable
        prev_unwrapped = None
        unwrapped = inspect.unwrap(self._reward_function)
        while unwrapped != prev_unwrapped:
            prev_unwrapped = unwrapped
            unwrapped = inspect.unwrap(unwrapped)
        return unwrapped

    @property
    def _reward_function_relative_path(self) -> str:
        """Get the relative path of the reward function"""
        return self._get_relative_path(self._reward_function_file_path)

    @property
    def _reward_function_file_path(self) -> str:
        """Get the file path of the reward function"""
        file_path = inspect.getsourcefile(self._unwrapped_reward_function)
        if not file_path:
            raise ValueError("Reward function must be defined in a file")
        return file_path

    @property
    def _reward_function_source(self) -> str:
        """Get the source code of the reward function itself only"""
        return inspect.getsource(self._unwrapped_reward_function)

    @property
    def _reward_function_file_source(self) -> str:
        """Get the source for the entire file that hosts the reward function"""
        with open(self._reward_function_file_path, "r", encoding="utf-8") as f:
            return f.read()

    @property
    def _reward_function_file_name(self) -> str:
        """Get the file name of the reward function"""
        file_name = os.path.basename(self._reward_function_file_path)
        return file_name

    def _validate_reward_function(self):
        """Validate that the reward function is properly decorated"""

        # if source does not decorate with @reward_function, raise an error
        if not self._reward_function_source.startswith("@reward_function"):
            raise ValueError("Reward function must be decorated with @reward_function")

    def _get_sibling_and_child_files(self) -> List[str]:
        """
        Recursively find all sibling and child files relative to the reward function file.
        Returns absolute paths of Python files (.py) in the same directory and subdirectories.

        Also ignore a default set of files that are probably not relevant to the
        reward function (hidden, node_modules, .git, etc.)
        """
        reward_function_dir = os.path.dirname(self._reward_function_file_path)
        all_files = []

        # Walk through the directory and all subdirectories
        for root, dirs, files in os.walk(reward_function_dir):
            # Skip hidden directories and common non-source directories
            dirs[:] = [d for d in dirs if not d.startswith(".") and d not in ["__pycache__", "node_modules", ".git"]]

            for file in files:
                if not file.startswith("."):
                    file_path = os.path.join(root, file)
                    all_files.append(file_path)

        return all_files

    def _get_relative_path(self, file_path: str) -> str:
        """
        Get the relative path of a file from the reward function's directory.

        Args:
            file_path: Absolute path to the file

        Returns:
            Relative path from the reward function's directory
        """
        reward_function_dir = os.path.dirname(self._reward_function_file_path)
        return os.path.relpath(file_path, reward_function_dir)

    def _get_file_source(self, file_path: str) -> str:
        """
        Read and return the source code of a file.

        Args:
            file_path: Absolute path to the file

        Returns:
            Source code of the file
        """
        try:
            with open(file_path, "r", encoding="utf-8") as f:
                return f.read()
        except Exception as e:
            logger.warning(f"Could not read file {file_path}: {e}")
            return ""

    def _convert_relative_imports(self, source_code: str, file_path: str) -> str:
        """
        Convert relative imports to absolute imports in Python source code.

        Args:
            source_code: The Python source code
            file_path: Absolute path to the file (used to determine package structure)

        Returns:
            Source code with relative imports converted to absolute imports
        """
        try:
            # Parse the source code into an AST
            tree = ast.parse(source_code)
        except SyntaxError:
            # If we can't parse the code, return it unchanged
            logger.warning(f"Could not parse Python code in {file_path}, skipping import conversion")
            return source_code

        # Get the package path for this file relative to the reward function directory
        reward_function_dir = os.path.dirname(self._reward_function_file_path)
        relative_file_path = os.path.relpath(file_path, reward_function_dir)

        # Convert file path to package path (remove .py extension and replace / with .)
        if relative_file_path.endswith(".py"):
            # Convert path to package notation
            package_path = relative_file_path[:-3].replace(os.sep, ".")
            # Remove __init__ from package path if present
            if package_path.endswith(".__init__"):
                package_path = package_path[:-9]
            current_package = package_path
        else:
            current_package = ""

        # Track modifications to make
        modifications = []

        # Walk through all import nodes
        for node in ast.walk(tree):
            if isinstance(node, (ast.ImportFrom, ast.Import)):
                if isinstance(node, ast.ImportFrom) and node.level > 0:
                    # This is a relative import
                    relative_level = node.level
                    module_name = node.module or ""

                    # Calculate the absolute module name
                    if current_package:
                        package_parts = current_package.split(".")
                        # Go up 'level' number of packages
                        if relative_level > len(package_parts):
                            # Can't go up more levels than we have, skip this import
                            continue

                        if relative_level == len(package_parts) + 1:
                            # Going to parent of root, assume empty base
                            absolute_module = module_name
                        else:
                            base_package_parts = (
                                package_parts[:-relative_level] if relative_level <= len(package_parts) else []
                            )
                            base_package = ".".join(base_package_parts) if base_package_parts else ""

                            if module_name:
                                absolute_module = f"{base_package}.{module_name}" if base_package else module_name
                            else:
                                absolute_module = base_package
                    else:
                        # We're at root level
                        absolute_module = module_name

                    # Store the modification info
                    modifications.append(
                        {
                            "lineno": node.lineno,
                            "col_offset": node.col_offset,
                            "original_level": relative_level,
                            "original_module": node.module,
                            "absolute_module": absolute_module,
                            "names": [alias.name for alias in node.names] if node.names else [],
                        }
                    )

        # Apply modifications by replacing lines
        if modifications:
            lines = source_code.splitlines()

            # Sort modifications by line number in reverse order to avoid offset issues
            modifications.sort(key=lambda x: x["lineno"], reverse=True)

            for mod in modifications:
                line_idx = mod["lineno"] - 1  # Convert to 0-based indexing
                if line_idx < len(lines):
                    original_line = lines[line_idx]

                    # Create the new import statement
                    if mod["names"]:
                        names_str = ", ".join(mod["names"])
                        if mod["absolute_module"]:
                            new_line = f"from {mod['absolute_module']} import {names_str}"
                        else:
                            # If absolute module is empty, convert to regular import
                            new_line = f"import {names_str}"
                    else:
                        if mod["absolute_module"]:
                            new_line = f"import {mod['absolute_module']}"
                        else:
                            # Skip empty imports
                            continue

                    # Preserve indentation from original line
                    original_indent = len(original_line) - len(original_line.lstrip())
                    new_line = " " * original_indent + new_line

                    lines[line_idx] = new_line

            return "\n".join(lines)

        return source_code

    def _get_all_file_contents(self) -> dict:
        """
        Get a dictionary mapping relative file paths to their source code
        for all sibling and child files of the reward function.
        Converts relative imports to absolute imports in the process.

        Returns:
            Dictionary with relative paths as keys and source code as values
        """
        file_contents = {}

        # Get all sibling and child files
        all_files = self._get_sibling_and_child_files()

        for file_path in all_files:
            relative_path = self._get_relative_path(file_path)
            source_code = self._get_file_source(file_path)
            # Convert relative imports to absolute imports
            converted_source = self._convert_relative_imports(source_code, file_path)
            file_contents[relative_path] = converted_source

        return file_contents

    def _create_evaluator_proto(self) -> SyncEvaluator:
        """Create the SyncEvaluator proto object from the reward function"""
        requirements = self._reward_function._reward_function_requirements

        # get dependencies
        if not requirements:
            file_path = inspect.getsourcefile(self._reward_function)
            dependencies_file_path = find_requirements_or_pyproject(file_path)
            requirements = get_requirements_from_pyproject_or_requirements_txt(dependencies_file_path)

        code_snippets = SyncCodeSnippets(
            file_contents=self._get_all_file_contents(),
            language="python",
            entry_file=self._reward_function_relative_path,
            entry_func=self._reward_function_name,
        )

        criteria = [
            SyncCriterion(
                name=self._reward_function.__name__,
                code_snippets=code_snippets,
                type=SyncCriterion.Type.CODE_SNIPPETS,
            )
        ]

        evaluator = SyncEvaluator(
            criteria=criteria,
            requirements=requirements,
            multi_metrics=True,
            rollup_settings=SyncRollupSettings(
                skip_rollup=True,
            ),
        )

        if self._reward_function.__doc__:
            evaluator.description = self._reward_function.__doc__.strip()

        return evaluator

    @property
    def name_prefix(self) -> str:
        """Get the name prefix of the evaluator"""
        id = self._reward_function._reward_function_id
        if not id:
            raise ValueError("Reward function must have an ID")
        return id

    def _list_existing_evaluators(self) -> List[SyncEvaluator]:
        """List all existing evaluators and filter by name prefix"""
        # Paginate through all evaluators
        all_evaluators: List[SyncEvaluator] = []
        page_token = None

        while True:
            list_evaluators_request = SyncListEvaluatorsRequest(
                page_size=200,
                page_token=page_token,
            )
            response = self._gateway.list_evaluators_sync(list_evaluators_request)

            all_evaluators.extend(response.evaluators)

            # Check if there are more pages
            if not response.next_page_token:
                break
            page_token = response.next_page_token

        # filter evaluators that start with name prefix, accounting for full paths
        evaluators = [
            evaluator for evaluator in all_evaluators if evaluator.name.split("/")[-1].startswith(self.name_prefix)
        ]

        return evaluators

    def _is_criterion_same(self, a: SyncCriterion, b: SyncCriterion) -> bool:
        """Compare two criteria for equality"""
        # recursively compare code_snippets
        if a.code_snippets.file_contents != b.code_snippets.file_contents:
            return False
        return True

    def _is_evaluator_same(self, a: SyncEvaluator, b: SyncEvaluator) -> bool:
        """Compare two evaluators for equality"""
        # recursively compare criteria
        if len(a.criteria) != len(b.criteria):
            return False
        if a.requirements != b.requirements:
            return False
        for a_criterion, b_criterion in zip(a.criteria, b.criteria):
            if not self._is_criterion_same(a_criterion, b_criterion):
                return False
        return True

    def preview(self, dataset) -> SyncPreviewEvaluatorResponse:
        sample_data = [json.dumps(j) for j in dataset]
        self.sync()
        if not self._proto:
            raise ValueError("Evaluator not synced. Something went wrong with the sync.")
        response = self._gateway.preview_evaluator_sync(
            SyncPreviewEvaluatorRequest(evaluator=self._proto, sample_data=sample_data)
        )
        return response

    def _extract_version_number(self, evaluator_name: str) -> int:
        """
        Extract the version number from an evaluator name.
        Returns 0 if no version is found.

        Args:
            evaluator_name: The full evaluator name

        Returns:
            The version number as an integer
        """
        evaluator_id = evaluator_name.split("/")[-1]

        if self.VERSION_DELIMITER in evaluator_id:
            try:
                version_str = evaluator_id.split(self.VERSION_DELIMITER)[-1]
                return int(version_str)
            except (ValueError, IndexError):
                return 0
        return 0

    def sync(self) -> "Evaluator":
        """
        Create or retrieve the evaluator on Fireworks.
        If an identical evaluator exists, reuse it. Otherwise, create a new versioned evaluator.
        """
        if self._proto is not None:
            logger.debug(f"Evaluator already synced: {self._proto.name}")
            return self

        # Create the evaluator proto
        evaluator_proto = self._create_evaluator_proto()

        # Get existing evaluators with matching name prefix
        existing_evaluators = self._list_existing_evaluators()

        # Check all existing evaluators for a match
        matching_evaluator = None
        for evaluator in existing_evaluators:
            if self._is_evaluator_same(evaluator, evaluator_proto):
                matching_evaluator = evaluator
                break

        if matching_evaluator:
            logger.debug(f"Using existing evaluator: {matching_evaluator.name}")
            self._proto = matching_evaluator
        else:
            # Find the latest version number to increment
            latest_version = 0
            for evaluator in existing_evaluators:
                version = self._extract_version_number(evaluator.name)
                latest_version = max(latest_version, version)

            # Create a new evaluator with incremented version
            evaluator_id = f"{self.name_prefix}{self.VERSION_DELIMITER}{latest_version + 1}"
            evaluator_proto.display_name = evaluator_id

            create_evaluator_request = SyncCreateEvaluatorRequest(
                evaluator=evaluator_proto,
                evaluator_id=evaluator_id,
            )

            logger.debug(f"Creating new evaluator: {evaluator_id}")
            self._proto = self._gateway.create_evaluator_sync(create_evaluator_request)

        return self

    @property
    def name(self) -> str:
        """Get the name of the evaluator"""
        if not self._proto:
            raise ValueError("Evaluator has no name. Make sure to run `sync` before calling this method.")
        return self._proto.name

    @property
    def id(self) -> str:
        """Get the ID of the evaluator"""
        if not self._proto:
            raise ValueError("Evaluator has no ID. Make sure to run `sync` before calling this method.")
        return self._proto.name.split("/")[-1]

    @property
    def url(self) -> str:
        """Get the URL to view this evaluator in the Fireworks dashboard"""
        return f"https://app.fireworks.ai/dashboard/evaluators/{self.id}"



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: gateway/workload.proto
# Protobuf Python Version: 5.29.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    0,
    '',
    'gateway/workload.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ..buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from . import deployment_pb2 as gateway_dot_deployment__pb2
from . import options_pb2 as gateway_dot_options__pb2
from . import status_pb2 as gateway_dot_status__pb2
from ..google.api import field_behavior_pb2 as google_dot_api_dot_field__behavior__pb2
from ..google.api import resource_pb2 as google_dot_api_dot_resource__pb2
from ..google.api import visibility_pb2 as google_dot_api_dot_visibility__pb2
from google.protobuf import field_mask_pb2 as google_dot_protobuf_dot_field__mask__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x16gateway/workload.proto\x12\x07gateway\x1a\x1b\x62uf/validate/validate.proto\x1a\x18gateway/deployment.proto\x1a\x15gateway/options.proto\x1a\x14gateway/status.proto\x1a\x1fgoogle/api/field_behavior.proto\x1a\x19google/api/resource.proto\x1a\x1bgoogle/api/visibility.proto\x1a google/protobuf/field_mask.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"\xfe\t\n\x08Workload\x12\x13\n\x04name\x18\x01 \x01(\tB\x05\xe2\x41\x02\x03\x05\x12!\n\x0c\x64isplay_name\x18\x02 \x01(\tB\x0b\xe2\x41\x01\x01\xbaH\x04r\x02\x18?\x12!\n\x0b\x64\x65scription\x18\x03 \x01(\tB\x0c\xe2\x41\x01\x01\xbaH\x05r\x03\x18\xe7\x07\x12\x36\n\x0b\x63reate_time\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x05\xe2\x41\x02\x03\x05\x12\x35\n\x0bupdate_time\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x04\xe2\x41\x01\x03\x12\x35\n\x0b\x65xpire_time\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x04\xe2\x41\x01\x01\x12\x35\n\npurge_time\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x05\xe2\x41\x02\x03\x05\x12\x36\n\x0b\x64\x65lete_time\x18\x08 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x05\xe2\x41\x02\x03\x05\x12+\n\ncreated_by\x18\t \x01(\tB\x17\xe2\x41\x02\x03\x05\xfa\xd2\xe4\x93\x02\x0c\x12\nOWNER_ONLY\x12-\n\x05state\x18\n \x01(\x0e\x32\x17.gateway.Workload.StateB\x05\xe2\x41\x02\x03\x05\x12&\n\x06status\x18\x0b \x01(\x0b\x32\x0f.gateway.StatusB\x05\xe2\x41\x02\x03\x05\x12S\n\x0b\x61nnotations\x18\x0c \x03(\x0b\x32\".gateway.Workload.AnnotationsEntryB\x1a\xe2\x41\x01\x01\xfa\xd2\xe4\x93\x02\x10\x12\x0eSUPERUSER_ONLY\x12,\n\tplacement\x18\r \x01(\x0b\x32\x12.gateway.PlacementB\x05\xe2\x41\x02\x01\x05\x12*\n\x11min_replica_count\x18\x0e \x01(\x05\x42\x0f\xe2\x41\x01\x01\xbaH\x04\x1a\x02(\x00\xa8\xf1\x04\x01\x12-\n\x11max_replica_count\x18\x0f \x01(\x05\x42\x12\xe2\x41\x01\x01\xbaH\x07\x1a\x05\x18\xe8\x07(\x00\xa8\xf1\x04\x01\x12\x44\n\x15\x64\x65sired_replica_count\x18\x10 \x01(\x05\x42%\xe2\x41\x01\x01\xbaH\x07\x1a\x05\x18\xe8\x07(\x00\xa8\xf1\x04\x01\xfa\xd2\xe4\x93\x02\r\x12\x0bOUTPUT_ONLY\x12=\n\x12\x61utoscaling_policy\x18\x11 \x01(\x0b\x32\x1a.gateway.AutoscalingPolicyB\x05\xe2\x41\x02\x01\x05\x12\x1d\n\x0eworkload_shape\x18\x12 \x01(\tB\x05\xe2\x41\x02\x02\x05\x12\x36\n\x12\x64isable_accounting\x18\x13 \x01(\x08\x42\x1a\xe2\x41\x01\x01\xfa\xd2\xe4\x93\x02\x10\x12\x0eSUPERUSER_ONLY\x12\x31\n\x0c\x66or_training\x18\x14 \x01(\x08\x42\x1b\xe2\x41\x02\x01\x05\xfa\xd2\xe4\x93\x02\x10\x12\x0eSUPERUSER_ONLY\x1a\x32\n\x10\x41nnotationsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"l\n\x05State\x12\x15\n\x11STATE_UNSPECIFIED\x10\x00\x12\x0c\n\x08\x43REATING\x10\x01\x12\t\n\x05READY\x10\x02\x12\x0c\n\x08\x44\x45LETING\x10\x03\x12\n\n\x06\x46\x41ILED\x10\x04\x12\x0c\n\x08UPDATING\x10\x05\x12\x0b\n\x07\x44\x45LETED\x10\x06:o\xea\x41]\n\x19\x61pi.fireworks.ai/Workload\x12+accounts/{AccountId}/workloads/{WorkloadId}*\tworkloads2\x08workload\x82\xf1\x04\x0b\n\x07\x41\x63\x63ount\x18\x01\"s\n\x15\x43reateWorkloadRequest\x12\x14\n\x06parent\x18\x01 \x01(\tB\x04\xe2\x41\x01\x02\x12)\n\x08workload\x18\x02 \x01(\x0b\x32\x11.gateway.WorkloadB\x04\xe2\x41\x01\x02\x12\x19\n\x0bworkload_id\x18\x03 \x01(\tB\x04\xe2\x41\x01\x01\"]\n\x12GetWorkloadRequest\x12\x12\n\x04name\x18\x01 \x01(\tB\x04\xe2\x41\x01\x02\x12\x33\n\tread_mask\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.FieldMaskB\x04\xe2\x41\x01\x01\"\x8b\x02\n\x14ListWorkloadsRequest\x12\x14\n\x06parent\x18\x01 \x01(\tB\x04\xe2\x41\x01\x02\x12\x17\n\tpage_size\x18\x02 \x01(\x05\x42\x04\xe2\x41\x01\x01\x12\x18\n\npage_token\x18\x03 \x01(\tB\x04\xe2\x41\x01\x01\x12\x14\n\x06\x66ilter\x18\x04 \x01(\tB\x04\xe2\x41\x01\x01\x12\x16\n\x08order_by\x18\x05 \x01(\tB\x04\xe2\x41\x01\x01\x12\x14\n\x0cshow_deleted\x18\x06 \x01(\x08\x12\x31\n\rshow_internal\x18\x07 \x01(\x08\x42\x1a\xe2\x41\x01\x01\xfa\xd2\xe4\x93\x02\x10\x12\x0eSUPERUSER_ONLY\x12\x33\n\tread_mask\x18\x08 \x01(\x0b\x32\x1a.google.protobuf.FieldMaskB\x04\xe2\x41\x01\x01\"j\n\x15ListWorkloadsResponse\x12$\n\tworkloads\x18\x01 \x03(\x0b\x32\x11.gateway.Workload\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t\x12\x12\n\ntotal_size\x18\x03 \x01(\x05\"y\n\x15UpdateWorkloadRequest\x12)\n\x08workload\x18\x01 \x01(\x0b\x32\x11.gateway.WorkloadB\x04\xe2\x41\x01\x02\x12\x35\n\x0bupdate_mask\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.FieldMaskB\x04\xe2\x41\x01\x01\"\\\n\x15\x44\x65leteWorkloadRequest\x12\x12\n\x04name\x18\x01 \x01(\tB\x04\xe2\x41\x01\x02\x12\x12\n\x04hard\x18\x02 \x01(\x08\x42\x04\xe2\x41\x01\x01\x12\x1b\n\rignore_checks\x18\x03 \x01(\x08\x42\x04\xe2\x41\x01\x01\x42\x43ZAgithub.com/fw-ai/fireworks/control_plane/protos/generated/gatewayb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'gateway.workload_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'ZAgithub.com/fw-ai/fireworks/control_plane/protos/generated/gateway'
  _globals['_WORKLOAD_ANNOTATIONSENTRY']._loaded_options = None
  _globals['_WORKLOAD_ANNOTATIONSENTRY']._serialized_options = b'8\001'
  _globals['_WORKLOAD'].fields_by_name['name']._loaded_options = None
  _globals['_WORKLOAD'].fields_by_name['name']._serialized_options = b'\342A\002\003\005'
  _globals['_WORKLOAD'].fields_by_name['display_name']._loaded_options = None
  _globals['_WORKLOAD'].fields_by_name['display_name']._serialized_options = b'\342A\001\001\272H\004r\002\030?'
  _globals['_WORKLOAD'].fields_by_name['description']._loaded_options = None
  _globals['_WORKLOAD'].fields_by_name['description']._serialized_options = b'\342A\001\001\272H\005r\003\030\347\007'
  _globals['_WORKLOAD'].fields_by_name['create_time']._loaded_options = None
  _globals['_WORKLOAD'].fields_by_name['create_time']._serialized_options = b'\342A\002\003\005'
  _globals['_WORKLOAD'].fields_by_name['update_time']._loaded_options = None
  _globals['_WORKLOAD'].fields_by_name['update_time']._serialized_options = b'\342A\001\003'
  _globals['_WORKLOAD'].fields_by_name['expire_time']._loaded_options = None
  _globals['_WORKLOAD'].fields_by_name['expire_time']._serialized_options = b'\342A\001\001'
  _globals['_WORKLOAD'].fields_by_name['purge_time']._loaded_options = None
  _globals['_WORKLOAD'].fields_by_name['purge_time']._serialized_options = b'\342A\002\003\005'
  _globals['_WORKLOAD'].fields_by_name['delete_time']._loaded_options = None
  _globals['_WORKLOAD'].fields_by_name['delete_time']._serialized_options = b'\342A\002\003\005'
  _globals['_WORKLOAD'].fields_by_name['created_by']._loaded_options = None
  _globals['_WORKLOAD'].fields_by_name['created_by']._serialized_options = b'\342A\002\003\005\372\322\344\223\002\014\022\nOWNER_ONLY'
  _globals['_WORKLOAD'].fields_by_name['state']._loaded_options = None
  _globals['_WORKLOAD'].fields_by_name['state']._serialized_options = b'\342A\002\003\005'
  _globals['_WORKLOAD'].fields_by_name['status']._loaded_options = None
  _globals['_WORKLOAD'].fields_by_name['status']._serialized_options = b'\342A\002\003\005'
  _globals['_WORKLOAD'].fields_by_name['annotations']._loaded_options = None
  _globals['_WORKLOAD'].fields_by_name['annotations']._serialized_options = b'\342A\001\001\372\322\344\223\002\020\022\016SUPERUSER_ONLY'
  _globals['_WORKLOAD'].fields_by_name['placement']._loaded_options = None
  _globals['_WORKLOAD'].fields_by_name['placement']._serialized_options = b'\342A\002\001\005'
  _globals['_WORKLOAD'].fields_by_name['min_replica_count']._loaded_options = None
  _globals['_WORKLOAD'].fields_by_name['min_replica_count']._serialized_options = b'\342A\001\001\272H\004\032\002(\000\250\361\004\001'
  _globals['_WORKLOAD'].fields_by_name['max_replica_count']._loaded_options = None
  _globals['_WORKLOAD'].fields_by_name['max_replica_count']._serialized_options = b'\342A\001\001\272H\007\032\005\030\350\007(\000\250\361\004\001'
  _globals['_WORKLOAD'].fields_by_name['desired_replica_count']._loaded_options = None
  _globals['_WORKLOAD'].fields_by_name['desired_replica_count']._serialized_options = b'\342A\001\001\272H\007\032\005\030\350\007(\000\250\361\004\001\372\322\344\223\002\r\022\013OUTPUT_ONLY'
  _globals['_WORKLOAD'].fields_by_name['autoscaling_policy']._loaded_options = None
  _globals['_WORKLOAD'].fields_by_name['autoscaling_policy']._serialized_options = b'\342A\002\001\005'
  _globals['_WORKLOAD'].fields_by_name['workload_shape']._loaded_options = None
  _globals['_WORKLOAD'].fields_by_name['workload_shape']._serialized_options = b'\342A\002\002\005'
  _globals['_WORKLOAD'].fields_by_name['disable_accounting']._loaded_options = None
  _globals['_WORKLOAD'].fields_by_name['disable_accounting']._serialized_options = b'\342A\001\001\372\322\344\223\002\020\022\016SUPERUSER_ONLY'
  _globals['_WORKLOAD'].fields_by_name['for_training']._loaded_options = None
  _globals['_WORKLOAD'].fields_by_name['for_training']._serialized_options = b'\342A\002\001\005\372\322\344\223\002\020\022\016SUPERUSER_ONLY'
  _globals['_WORKLOAD']._loaded_options = None
  _globals['_WORKLOAD']._serialized_options = b'\352A]\n\031api.fireworks.ai/Workload\022+accounts/{AccountId}/workloads/{WorkloadId}*\tworkloads2\010workload\202\361\004\013\n\007Account\030\001'
  _globals['_CREATEWORKLOADREQUEST'].fields_by_name['parent']._loaded_options = None
  _globals['_CREATEWORKLOADREQUEST'].fields_by_name['parent']._serialized_options = b'\342A\001\002'
  _globals['_CREATEWORKLOADREQUEST'].fields_by_name['workload']._loaded_options = None
  _globals['_CREATEWORKLOADREQUEST'].fields_by_name['workload']._serialized_options = b'\342A\001\002'
  _globals['_CREATEWORKLOADREQUEST'].fields_by_name['workload_id']._loaded_options = None
  _globals['_CREATEWORKLOADREQUEST'].fields_by_name['workload_id']._serialized_options = b'\342A\001\001'
  _globals['_GETWORKLOADREQUEST'].fields_by_name['name']._loaded_options = None
  _globals['_GETWORKLOADREQUEST'].fields_by_name['name']._serialized_options = b'\342A\001\002'
  _globals['_GETWORKLOADREQUEST'].fields_by_name['read_mask']._loaded_options = None
  _globals['_GETWORKLOADREQUEST'].fields_by_name['read_mask']._serialized_options = b'\342A\001\001'
  _globals['_LISTWORKLOADSREQUEST'].fields_by_name['parent']._loaded_options = None
  _globals['_LISTWORKLOADSREQUEST'].fields_by_name['parent']._serialized_options = b'\342A\001\002'
  _globals['_LISTWORKLOADSREQUEST'].fields_by_name['page_size']._loaded_options = None
  _globals['_LISTWORKLOADSREQUEST'].fields_by_name['page_size']._serialized_options = b'\342A\001\001'
  _globals['_LISTWORKLOADSREQUEST'].fields_by_name['page_token']._loaded_options = None
  _globals['_LISTWORKLOADSREQUEST'].fields_by_name['page_token']._serialized_options = b'\342A\001\001'
  _globals['_LISTWORKLOADSREQUEST'].fields_by_name['filter']._loaded_options = None
  _globals['_LISTWORKLOADSREQUEST'].fields_by_name['filter']._serialized_options = b'\342A\001\001'
  _globals['_LISTWORKLOADSREQUEST'].fields_by_name['order_by']._loaded_options = None
  _globals['_LISTWORKLOADSREQUEST'].fields_by_name['order_by']._serialized_options = b'\342A\001\001'
  _globals['_LISTWORKLOADSREQUEST'].fields_by_name['show_internal']._loaded_options = None
  _globals['_LISTWORKLOADSREQUEST'].fields_by_name['show_internal']._serialized_options = b'\342A\001\001\372\322\344\223\002\020\022\016SUPERUSER_ONLY'
  _globals['_LISTWORKLOADSREQUEST'].fields_by_name['read_mask']._loaded_options = None
  _globals['_LISTWORKLOADSREQUEST'].fields_by_name['read_mask']._serialized_options = b'\342A\001\001'
  _globals['_UPDATEWORKLOADREQUEST'].fields_by_name['workload']._loaded_options = None
  _globals['_UPDATEWORKLOADREQUEST'].fields_by_name['workload']._serialized_options = b'\342A\001\002'
  _globals['_UPDATEWORKLOADREQUEST'].fields_by_name['update_mask']._loaded_options = None
  _globals['_UPDATEWORKLOADREQUEST'].fields_by_name['update_mask']._serialized_options = b'\342A\001\001'
  _globals['_DELETEWORKLOADREQUEST'].fields_by_name['name']._loaded_options = None
  _globals['_DELETEWORKLOADREQUEST'].fields_by_name['name']._serialized_options = b'\342A\001\002'
  _globals['_DELETEWORKLOADREQUEST'].fields_by_name['hard']._loaded_options = None
  _globals['_DELETEWORKLOADREQUEST'].fields_by_name['hard']._serialized_options = b'\342A\001\001'
  _globals['_DELETEWORKLOADREQUEST'].fields_by_name['ignore_checks']._loaded_options = None
  _globals['_DELETEWORKLOADREQUEST'].fields_by_name['ignore_checks']._serialized_options = b'\342A\001\001'
  _globals['_WORKLOAD']._serialized_start=292
  _globals['_WORKLOAD']._serialized_end=1570
  _globals['_WORKLOAD_ANNOTATIONSENTRY']._serialized_start=1297
  _globals['_WORKLOAD_ANNOTATIONSENTRY']._serialized_end=1347
  _globals['_WORKLOAD_STATE']._serialized_start=1349
  _globals['_WORKLOAD_STATE']._serialized_end=1457
  _globals['_CREATEWORKLOADREQUEST']._serialized_start=1572
  _globals['_CREATEWORKLOADREQUEST']._serialized_end=1687
  _globals['_GETWORKLOADREQUEST']._serialized_start=1689
  _globals['_GETWORKLOADREQUEST']._serialized_end=1782
  _globals['_LISTWORKLOADSREQUEST']._serialized_start=1785
  _globals['_LISTWORKLOADSREQUEST']._serialized_end=2052
  _globals['_LISTWORKLOADSRESPONSE']._serialized_start=2054
  _globals['_LISTWORKLOADSRESPONSE']._serialized_end=2160
  _globals['_UPDATEWORKLOADREQUEST']._serialized_start=2162
  _globals['_UPDATEWORKLOADREQUEST']._serialized_end=2283
  _globals['_DELETEWORKLOADREQUEST']._serialized_start=2285
  _globals['_DELETEWORKLOADREQUEST']._serialized_end=2377
# @@protoc_insertion_point(module_scope)



from typing_extensions import Literal, Required, TypedDict, Union
from openai.types.chat.completion_create_params import ResponseFormat as OpenAIResponseFormat


class ResponseFormatGrammar(TypedDict, total=False):
    type: Required[Literal["grammar"]]
    """The type of response format being defined. Always `text`."""

    grammar: str
    """See https://fireworks.ai/docs/structured-responses/structured-output-grammar-based"""


ResponseFormat = Union[OpenAIResponseFormat, ResponseFormatGrammar]



from typing import Literal


AcceleratorTypeLiteral = Literal[
    "ACCELERATOR_TYPE_UNSPECIFIED",
    "NVIDIA_A100_80GB",
    "NVIDIA_H100_80GB",
    "AMD_MI300X_192GB",
    "NVIDIA_A10G_24GB",
    "NVIDIA_A100_40GB",
    "NVIDIA_L4_24GB",
    "NVIDIA_H200_141GB",
    "NVIDIA_B200_180GB",
]


RegionLiteral = Literal[
    "REGION_UNSPECIFIED",
    "US_IOWA_1",
    "US_VIRGINIA_1",
    "US_VIRGINIA_2",
    "US_ILLINOIS_1",
    "AP_TOKYO_1",
    "EU_LONDON_1",
    "US_ARIZONA_1",
    "US_TEXAS_1",
    "US_ILLINOIS_2",
    "EU_FRANKFURT_1",
    "US_TEXAS_2",
    "EU_PARIS_1",
    "EU_HELSINKI_1",
    "US_NEVADA_1",
    "EU_ICELAND_1",
    "EU_ICELAND_2",
    "US_WASHINGTON_1",
    "US_WASHINGTON_2",
    "EU_ICELAND_DEV_1",
]

MultiRegionLiteral = Literal["MULTI_REGION_UNSPECIFIED", "GLOBAL", "US"]


ReasoningEffort = Literal["low", "medium", "high"]

"""
serverless: always use serverless, if not available, raise an error
on-demand: always use on-demand. For LoRA addons, this will create a new on-demand deployment for the LoRA model without addons enabled (for performance reasons).
auto: use serverless if available, otherwise use on-demand. For LoRA addons, this will create a new on-demand deployment for the LoRA model with addons enabled.
on-demand-lora: always use on-demand (even if serverless is available). For LoRA addons, this will create a new on-demand deployment for the LoRA model with addons enabled.
"""
DeploymentTypeLiteral = Literal["serverless", "on-demand", "auto", "on-demand-lora"]


DeploymentStrategyLiteral = Literal["serverless", "on-demand", "serverless-lora", "on-demand-lora"]

PrecisionLiteral = Literal[
    "FP16",
    "FP8",
    "FP8_MM",
    "FP8_AR",
    "FP8_MM_KV_ATTN",
    "FP8_KV",
    "FP8_MM_V2",
    "FP8_V2",
    "FP8_MM_KV_ATTN_V2",
    "NF4",
]

DirectRouteTypeLiteral = Literal["INTERNET", "GCP_PRIVATE_SERVICE_CONNECT", "AWS_PRIVATELINK"]


WeightPrecisionLiteral = Literal["WEIGHT_PRECISION_UNSPECIFIED", "BFLOAT16", "INT8", "NF4"]



# Copyright (c) Fireworks AI, Inc. and affiliates.
#
# All Rights Reserved.

from typing import Optional

from fireworks._const import FIREWORKS_API_BASE_URL, FIREWORKS_GATEWAY_ADDR
from fireworks._util import get_api_key_from_env
from fireworks.platform.gateway_wrapper import GatewayWrapper
from fireworks.control_plane.generated.protos.gateway import GatewayStub
import grpc
from grpclib.client import Channel
import httpx
from functools import cache as sync_cache


class FireworksPlatform:
    """
    Platform client that exposes all its endpoints.
    """

    def __init__(
        self,
        *,
        server_addr: str = FIREWORKS_GATEWAY_ADDR,
        api_key: Optional[str] = None,
    ) -> None:
        """
        Args:
            server_addr: the network address of the gateway server.
            api_key: the API key to use for authentication.
        """
        self._server_addr = server_addr
        if not api_key:
            api_key = get_api_key_from_env()
            if not api_key:
                raise ValueError(
                    "Fireworks API key not found. Please provide an API key either as a parameter "
                    "or by setting the FIREWORKS_API_KEY environment variable. "
                    "You can create a new API key at https://fireworks.ai/settings/users/api-keys or "
                    "by using `firectl create api-key --key-name <key-name>` command."
                )
        self._api_key = api_key
        self._host = self._server_addr.split(":")[0]
        self._port = int(self._server_addr.split(":")[1])
        self._channel = Channel(host=self._host, port=self._port, ssl=True)
        self.client = GatewayWrapper(self._channel, metadata=[("x-api-key", api_key)], account_id=self.account_id())

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        self._channel.close()

    @sync_cache
    def account_id(self) -> str:
        # make curl -v -H "Authorization: Bearer XXX" https://api.fireworks.ai/verifyApiKey
        # and read x-fireworks-account-id from headers of the response
        with httpx.Client() as client:
            response = client.get(
                f"{FIREWORKS_API_BASE_URL}/verifyApiKey",
                headers={
                    "Authorization": f"Bearer {self._api_key}",
                },
            )
            return response.headers["x-fireworks-account-id"]
